/***************************************************************************
 * Copyright 2001-2006 The eXo Platform SARL         All rights reserved.  *
 * Please look at license.txt in info directory for more license detail.   *
 **************************************************************************/

package org.exoplatform.applications.ooplugin;

import java.io.File;
import java.io.FileOutputStream;
import java.util.ArrayList;

import org.exoplatform.applications.ooplugin.dialog.DialogBuilder;
import org.exoplatform.applications.ooplugin.dialog.EventHandler;
import org.exoplatform.frameworks.webdavclient.Const;
import org.exoplatform.frameworks.webdavclient.commands.DavGet;

import com.sun.star.awt.Rectangle;
import com.sun.star.awt.WindowAttribute;
import com.sun.star.awt.WindowClass;
import com.sun.star.awt.WindowDescriptor;
import com.sun.star.awt.XControlContainer;
import com.sun.star.awt.XDialog;
import com.sun.star.awt.XMessageBox;
import com.sun.star.awt.XToolkit;
import com.sun.star.awt.XWindowPeer;
import com.sun.star.frame.XFrame;
import com.sun.star.lang.XComponent;
import com.sun.star.uno.UnoRuntime;
import com.sun.star.uno.XComponentContext;

/**
 * Created by The eXo Platform SARL
 * Author : Vitaly Guly <gavrik-vetal@ukr.net/mail.ru>
 * @version $Id: $
 */

public class PlugInDialog {

  protected String dialogName = "";
  protected boolean enabled = false;
  
  protected XComponentContext xComponentContext;  
  protected XFrame xFrame;
  protected XToolkit xToolkit;
  
  protected XDialog xDialog;
  protected XControlContainer xControlContainer;
  
  protected ArrayList<EventHandler> eventHandlers = new ArrayList<EventHandler>();
  
  protected WebDavConfig config;
  
  public PlugInDialog(WebDavConfig config, XComponentContext xComponentContext, XFrame xFrame, XToolkit xToolkit) {
    this.config = config;
    this.xComponentContext = xComponentContext;
    this.xFrame = xFrame;
    this.xToolkit = xToolkit;
  }
  
  public void setEnabled(boolean enabled) {
    this.enabled = enabled;
  }
  
  public boolean isEnableg() {
    return enabled;
  }
  
  public String getDialogName() {
    return dialogName;
  }
  
  public XControlContainer getControlContainer() {
    return xControlContainer;
  }
  
  public void setControlContainer(XControlContainer xControlContainer) {
    this.xControlContainer = xControlContainer;
  }
  
  public XComponentContext getConponentContext() {
    return xComponentContext;
  }
  
  public void addHandler(String componentName, int componentType, Object listener) {    
    EventHandler handler = new EventHandler(componentName, componentType, listener);
    eventHandlers.add(handler);
  }
  
  public boolean launchBeforeOpen() {
    return true;
  }
  
  public void createDialog() throws com.sun.star.uno.Exception {
    if ("".equals(dialogName)) {
      return;
    }
    
    try {
      DialogBuilder builder = new DialogBuilder(this, xFrame, xToolkit);
      builder.init();
      Object dialog = builder.createDialog(dialogName, eventHandlers);
      
      xDialog = (XDialog)UnoRuntime.queryInterface(XDialog.class, dialog);
      
      if (launchBeforeOpen()) {
        enabled = true;
        xDialog.execute();
      }
      
      XComponent xComponent = (XComponent)UnoRuntime.queryInterface(XComponent.class, dialog);
      xComponent.dispose();
      
    } catch (java.lang.Exception exc) {
      Log.info("Unhandled exception. " + exc.getMessage(), exc);
    }    
    
  }
  
  protected void prepareTmpPath(String tempPath) {    
    String documentPath = LocalFileSystem.getDocumentsPath() + File.separatorChar + "repository" + tempPath;
    documentPath = documentPath.replace("\\", "/");
    File outDirectory = new File(documentPath);
    if (!outDirectory.exists()) {
      outDirectory.mkdirs(); 
    }
    
  }  
  
  protected void doOpenRemoteFile(String href) throws Exception {
    String serverPrefix = config.getContext().getServerPrefix();
    if (!href.startsWith(serverPrefix)) {
      return;
    }
    
    String resourcePath = href.substring(serverPrefix.length());
    if (!resourcePath.startsWith("/")) {
      resourcePath = "/" + resourcePath;
    }

    DavGet davGet = new DavGet(config.getContext());
    davGet.setResourcePath(resourcePath);
    int status = davGet.execute();
    
    if (status != Const.HttpStatus.OK) {
      showMessageBox("Can't open remote file. ErrorCode: " + status);
      return;
    }
    
    prepareTmpPath(resourcePath);

    String filePath = LocalFileSystem.getDocumentsPath() + File.separatorChar + "repository" + resourcePath;
    filePath = filePath.replace("\\", "/");
    
    filePath = filePath.replace("?", ".");
    
    File outFile = new File(filePath);
    if (outFile.exists()) {
      outFile.delete();
    }
    
    outFile.createNewFile();
    FileOutputStream fileOutStream = new FileOutputStream(outFile);
    
    byte []fileContent = davGet.getResponseDataBuffer();
    fileOutStream.write(fileContent);
    fileOutStream.close();
    
    //XComponent component = OOUtils.loadFromFile(xComponentContext, filePath, resourcePath);
    OOUtils.loadFromFile(xComponentContext, filePath, resourcePath);    
  }  
  
  public void showMessageBox(String sMessage) {
    try {
        if (null != xFrame && null != xToolkit) {
            WindowDescriptor aDescriptor = new WindowDescriptor();
            aDescriptor.Type              = WindowClass.MODALTOP;
            aDescriptor.WindowServiceName = new String("infobox");
            aDescriptor.ParentIndex       = -1;
            aDescriptor.Parent            = (XWindowPeer)UnoRuntime.queryInterface(
                XWindowPeer.class, xFrame.getContainerWindow());
            aDescriptor.Bounds            = new Rectangle(0,0,300,200);
            aDescriptor.WindowAttributes  = WindowAttribute.BORDER |
            WindowAttribute.MOVEABLE |
            WindowAttribute.CLOSEABLE;
            
            XWindowPeer xPeer = xToolkit.createWindow(aDescriptor);
            if (null != xPeer) {
              XMessageBox xMsgBox = (XMessageBox)UnoRuntime.queryInterface(
                  XMessageBox.class, xPeer);
              if (null != xMsgBox)
              {
                xMsgBox.setCaptionText("eXo-Platform OOPlug-In.");
                xMsgBox.setMessageText(sMessage);
                xMsgBox.execute();
              }
            }
        }
    } catch (com.sun.star.uno.Exception e) {
    	Log.info("Unhandled exception", e);
      // do your error handling 
    }
  }  
  
}
